# Copyright (c) Stas Shtin, 2010

# This file is part of IPyPBX.

# IPyPBX is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# IPyPBX is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with IPyPBX.  If not, see <http://www.gnu.org/licenses/>.

#from ipypbx import models
from PyQt4 import QtCore, QtGui, QtSql


class BaseController(QtCore.QObject):
    """
    Base class for other controllers.

    Doesn't do anything useful on its own.
    """
    fields = ()
    hidden_fields = ()
    visible_fields = ()
    
    def __init__(self, model=None, view_list=None, view_display=None, parent=None):
        super(BaseController, self).__init__(parent)
        
        # Find out base name.
        classname = self.__class__.__name__
        self.base_name = (
            classname[:-10] if classname.endswith('Controller')
            else classname)
        self.base_name = self.base_name[0].lower() + self.base_name[1:]

        if model:
            # We're given an existing model.
            self.model = model
        else:
            # Initialize a new model.
            self.model = QtSql.QSqlTableModel(parent)
            self.model.setTable(self.base_name + 's')

            # Create model header from fields list.
            for i, field in enumerate(self.fields):
                self.model.setHeaderData(
                    i, QtCore.Qt.Horizontal, QtCore.QVariant(field))

            # Fetch model data.
            self.model.select()

        if view_list:
            # We're given an existing view list.
            self.view_list = view_list
        else:
            # Get view list from the parent.
            self.view_list = getattr(parent, self.base_name + 'ViewList')
            self.view_list.setModel(self.model)

            # Hide fields not meant for display.
            for i, field in enumerate(self.fields):
                if (field in self.hidden_fields or
                    field not in self.visible_fields):
                    self.view_list.setColumnHidden(i, True)

            # Stretch headers to fill all available width.
            self.view_list.setSelectionMode(QtGui.QTableView.SingleSelection)
            self.view_list.setSelectionBehavior(QtGui.QTableView.SelectRows)
            self.view_list.resizeColumnsToContents()
            self.view_list.resizeRowsToContents()
            self.view_list.horizontalHeader().setStretchLastSection(True)

        if view_display:
            # We're given an existing view display.
            self.view_display = view_display
        else:
            # Get view display from the parent.
            self.view_display = getattr(parent, self.base_name + 'ViewDisplay')
            self.view_display.setModel(self.model)

        # Register signals for this controller.
        for sender, signal, receiver in self.getSignalsData():
            QtCore.QObject.connect(sender, QtCore.SIGNAL(signal), receiver)        

    def getSignalsData(self):
        """
        Default signals built from controller's base name.
        """
        parent = self.parent()
        
        return (
            (getattr(parent, self.base_name + 'Add'), 'clicked()', self.add),
            (getattr(parent, self.base_name + 'ViewList'),
             'currentRowChanged(int)', self.select),
            (getattr(parent, self.base_name + 'Save'), 'clicked()',
             self.save),
            )

    def add(self):
        """
        TODO: Default implementation.
        """
        return NotImplemented
    
    def save(self):
        """
        TODO: Default implementation.
        """
        return NotImplemented

    def add(self):
        """
        TODO: Default implementation.
        """
        return NotImplemented


class ConnectionController(BaseController):
    """
    Connections handler.
    """
    fields = (
        'ID', 'Name', 'Local IP Address', 'Local Port',
        'Freeswitch IP Address', 'Freeswitch Port')
    visible_fields = ('Name', 'Freeswitch IP Address', 'Freeswitch Port')
    
    def select(self, row):
        """
        Select another connection as current.
        """
        self.currentConnection = self.connections[row]

        # Fill in form based on selection.
        self.parent.ui.connectionName.setText(self.currentConnection.name)
        self.parent.ui.connectionLocalIpAddress.setText(
            self.currentConnection.local_ip_address)
        self.parent.ui.connectionLocalPort.setText(
            unicode(self.currentConnection.local_port))
        self.parent.ui.connectionFreeswitchIpAddress.setText(
            self.currentConnection.freeswitch_ip_address)
        self.parent.ui.connectionFreeswitchPort.setText(
            unicode(self.currentConnection.freeswitch_port))

    def clone(self):
        """
        TODO: Clone an existing connection.

        This creates a new connection with bound data copied from another one.
        """

    def add(self):
        """
        Add new connection.
        """
        self.currentConnection = None

        name_template = 'New connection [{0:02}]'
        for i in xrange(1, 100):
            name = name_template.format(i)
            connection_exists = False
            for connection in self.connections:
                if connection.name == name:
                    connection_exists = True
                    break

            if not connection_exists:
                break
            
        self.parent.ui.connectionName.setText('New connection')
        self.parent.ui.connectionName.setFocus()
        self.parent.ui.connectionName.selectAll()
        self.parent.ui.connectionLocalIpAddress.clear()
        self.parent.ui.connectionLocalPort.clear()
        self.parent.ui.connectionFreeswitchIpAddress.clear()
        self.parent.ui.connectionFreeswitchPort.clear()

    def save(self):
        """
        Save new or existing connection.
        """
        name = unicode(self.parent.ui.connectionName.text())

        # Add to connection list if we've created it.
        if self.currentConnection is None:            
            #self.currentConnection = models.Connection(store=state.store)            
            self.connections.append(self.currentConnection)
            self.parent.ui.connectionList.addItem(name)

        self.currentConnection.name = name
        self.currentConnection.local_ip_address = unicode(
            self.parent.ui.connectionLocalIpAddress.text())
        self.currentConnection.local_port = int(
            self.parent.ui.connectionLocalPort.text())
        self.currentConnection.freeswitch_ip_address = unicode(
            self.parent.ui.connectionFreeswitchIpAddress.text())
        self.currentConnection.freeswitch_port = int(
            self.parent.ui.connectionFreeswitchPort.text())

        self.currentConnection.checkpoint()
